@@ -8,38 +8,37 @@
 #'
 #' @param data Numeric data matrix.
 #' @param init_mean Optional initial mean vector.
-#' @param fix_mean Fix the means to their starting value? Must initialize
+#' @param fix_mean Fix the mean to its starting value? Initial values must be
+#'   provided if \code{TRUE}.
+#' @param lambda Optional ridge term added to covariance matrix to ensure 
+#'   positive definiteness.
 #' @return An object of class \code{mvn}.
 #' @noRd
 
 FitMVNComplete <- function(
   data,
   init_mean = NULL,
-  fix_mean = FALSE
+  fix_mean = FALSE,
+  lambda = 0
 ) {
   
-  # Dimensions.
   n <- nrow(data)
   d <- ncol(data)
   orig_col_names <- colnames(data)
-  
-  # Mean.
+
   if (fix_mean) {
     new_mean <- init_mean
   } else {
     new_mean <- apply(data, 2, mean)
   }
-  
-  # Covariance.
-  new_cov <- matCov(data, data)
-  new_cov_inv <- matInv(new_cov)
+
+  new_cov <- matCov(data, data, eps = lambda)
   dimnames(new_cov) <- list(orig_col_names, orig_col_names)
-  
-  # Objective (log-likelihood).
-  n <- nrow(data)
+
   mean_mat <- matrix(data = new_mean, nrow = n, ncol = d, byrow = TRUE)
   resid <- data - mean_mat
-  objective <- -n * log(det(new_cov)) - tr(MMP(matInv(new_cov), matIP(resid, resid)))
+  objective <- -n * matDet(new_cov, logDet = TRUE) - 
+    tr(MMP(matInv(new_cov), matIP(resid, resid)))
   
   # Output.
   out <- methods::new(
@@ -61,13 +60,16 @@ FitMVNComplete <- function(
 #' @param data_comp Complete cases. 
 #' @param init_mean Optional initial mean vector.
 #' @param init_cov Optional initial covariance matrix.
+#' @param lambda Optional ridge term added to covariance matrix to ensure 
+#'   positive definiteness.
 #' @return List containing the initialized `mean` and `cov`. 
 #' @noRd
 
 MVNMissInit <- function(
   data_comp,
   init_mean,
-  init_cov
+  init_cov,
+  lambda = 0
 ) {
   
   # Output structure. 
@@ -80,7 +82,7 @@ MVNMissInit <- function(
     theta0$mean <- init_mean
     theta0$cov <- init_cov
     
-  # Case 2: At least one of init_means or S0 is null
+  # Case 2: At least one of init_mean or init_cov is null
   } else {
     
     # Check for complete obs
@@ -97,7 +99,7 @@ MVNMissInit <- function(
     
     # Initialize covariance if null.
     if (is.null(init_cov)) {
-      theta0$cov <- matCov(data_comp, data_comp)
+      theta0$cov <- matCov(data_comp, data_comp, eps = lambda)
     } else {
       theta0$cov <- init_cov
     }
@@ -114,7 +116,10 @@ MVNMissInit <- function(
 #' 
 #' @param split_data Data partitioned by missingness.
 #' @param theta List containing the current `mean` and `cov`. 
-#' @param fix_mean Fix the mean to its starting value? Must initialize. 
+#' @param fix_mean Fix the mean to its starting value? Initial values must be
+#'   provided if \code{TRUE}. 
+#' @param lambda Optional ridge term added to covariance matrix to ensure 
+#'   positive definiteness.
 #' @return List containing:
 #' \itemize{
 #'   \item The updated `mean` and `cov`.
@@ -126,7 +131,8 @@ MVNMissInit <- function(
 MVNMissUpdate <- function(
   split_data,
   theta,
-  fix_mean = FALSE
+  fix_mean = FALSE,
+  lambda = 0
 ) {
   
   # Unpack
@@ -162,7 +168,8 @@ MVNMissUpdate <- function(
   }
  
   ## Initial EM objective. 
-  old_obj <- (-n) * log(det(old_cov)) - tr(MMP(matInv(old_cov), resid_cov))
+  old_obj <- (-n) * matDet(old_cov, logDet = TRUE) - 
+    tr(MMP(matInv(old_cov), resid_cov))
   
   # Update mean:
   if (fix_mean) {
@@ -186,7 +193,7 @@ MVNMissUpdate <- function(
   }
   
   # Update covariance.
-  new_cov <- array(0, c(d, d))
+  new_cov <- lambda * diag(d)
   
   ## Complete observations.
   if (n0 > 0) {
@@ -207,11 +214,11 @@ MVNMissUpdate <- function(
   }
   
   ## Normalize covariance matrix.
-  new_cov <- (new_cov / n)
+  new_cov <- (new_cov / n) 
   dimnames(new_cov) <- list(split_data$orig_col_names, split_data$orig_col_names)
   
   ## Final EM objective.
-  new_obj <- (-n) * log(det(new_cov)) - (d * n)
+  new_obj <- (-n) * matDet(new_cov, logDet = TRUE) - (d * n)
   
   # Increment.
   delta <- new_obj - old_obj
@@ -260,13 +267,13 @@ Maximization <- function(
     if (theta1$delta > 0) {
       theta0 <- theta1
       if (report) {
-        cat("Objective increment: ", signif(theta1$d, digits = 3), "\n")
+        cat("Objective increment: ", signif(theta1$delta, digits = 3), "\n")
       }
     }
     
     # Terminate if increment is below tolerance.
     if (theta1$delta < eps) {
-      # If EM failes to perform any updates, e.g. the iteration is initialized
+      # If EM fails to perform any updates, e.g. the iteration is initialized
       # at the MLE, keep initial objective.
       if (i == 1) {
         theta0$new_obj <- theta1$old_obj
@@ -295,46 +302,26 @@ Maximization <- function(
 #' 
 #' @param split_data Data partitioned by missingness.
 #' @param theta List containing the `mean` and `cov`.
-#' @return Data.matrix, in the same order as the original data, with missing values
-#'   imputed to their expectations. 
+#' @return Numeric matrix in the same row order as the original data, with
+#'   missing values imputed to their posterior expectations. 
 #' @noRd
 
-MVNMissImpute <- function(
-  split_data,
-  theta
-) {
-  
-  d <- split_data$n_col 
-  out <- matrix(NA, nrow = 0, ncol = d)
-  
-  # Complete data.
+MVNMissImpute <- function(split_data, theta) {
+  d <- split_data$n_col
   n0 <- split_data$n0
-  if (n0 > 0) {
-    out <- rbind(out, split_data$data_comp)
-  }
-  
-  # Impute data.
   n1 <- split_data$n1
-  if (n1 > 0) {
-    data_imp <- WorkResp(split_data$data_incomp, theta$mean, theta$cov)
-    out <- rbind(out, data_imp)
-  }
-  
-  # Empty data.
   n2 <- split_data$n2
-  if (n2 > 0) {
-    data_empty <- matrix(data = theta$mean, nrow = n2, ncol = d, byrow = TRUE)
-    out <- rbind(out, data_empty)
-  }
-  
-  # Restore initial order. 
-  init_order <- split_data$init_order
-  out <- out[order(init_order), , drop = FALSE]
-  
-  # Output
-  rownames(out) <- split_data$orig_row_names
-  colnames(out) <- split_data$orig_col_names
-  return(out)
+
+  out <- list(
+    data_comp    = if (n0 > 0) split_data$data_comp else matrix(NA, 0L, d),
+    data_incomp  = if (n1 > 0) WorkResp(split_data$data_incomp, theta$mean, theta$cov) else matrix(NA, 0L, d),
+    data_empty   = if (n2 > 0) matrix(theta$mean, n2, d, byrow = TRUE) else matrix(NA, 0L, d)
+  )
+  split_out <- split_data
+  split_out$data_comp   <- out$data_comp
+  split_out$data_incomp <- out$data_incomp
+  split_out$data_empty  <- out$data_empty
+  ReconstituteData(split_out)
 }
 
 
@@ -344,8 +331,11 @@ MVNMissImpute <- function(
 #'
 #' @param data Numeric data matrix.
 #' @param init_mean Optional initial mean vector.
-#' @param fix_mean Fix the means to their starting value? Must initialize.
+#' @param fix_mean Fix the mean to its starting value? Initial values must be
+#'   provided if \code{TRUE}.
 #' @param init_cov Optional initial covariance matrix.
+#' @param lambda Optional ridge term added to covariance matrix to ensure 
+#'   positive definiteness.
 #' @param maxit Maximum number of EM iterations.
 #' @param eps Minimum acceptable increment in the EM objective.
 #' @param report Report fitting progress?
@@ -354,12 +344,13 @@ MVNMissImpute <- function(
 
 FitMVNMiss <- function(
   data,
-  init_mean,
-  fix_mean,
-  init_cov,
-  maxit,
-  eps,
-  report 
+  init_mean = NULL, 
+  fix_mean = FALSE, 
+  init_cov = NULL, 
+  lambda = 0.0,
+  maxit = 100, 
+  eps = 1e-6, 
+  report = TRUE
 ) {
   
   # Partitioned data
@@ -369,11 +360,12 @@ FitMVNMiss <- function(
   theta0 <- MVNMissInit(
     split_data$data_comp,
     init_mean,
-    init_cov
+    init_cov,
+    lambda = lambda
   )
   
   # Maximization
-  Update <- function(theta) {MVNMissUpdate(split_data, theta, fix_mean)}
+  Update <- function(theta) {MVNMissUpdate(split_data, theta, fix_mean, lambda)}
   theta1 <- Maximization(theta0, Update, maxit, eps, report)
   
   # Imputation.
@@ -404,8 +396,11 @@ FitMVNMiss <- function(
 #'
 #' @param data Numeric data matrix.
 #' @param init_mean Optional initial mean vector.
-#' @param fix_mean Fix the mean to its starting value? Must initialize. 
+#' @param fix_mean Fix the mean to its starting value? Initial values must be
+#'   provided if \code{TRUE}. 
 #' @param init_cov Optional initial covariance matrix.
+#' @param lambda Optional ridge term added to covariance matrix to ensure 
+#'   positive definiteness.
 #' @param maxit Maximum number of EM iterations.
 #' @param eps Minimum acceptable increment in the EM objective.
 #' @param report Report fitting progress?
@@ -416,6 +411,7 @@ FitMVN <- function(
   init_mean = NULL, 
   fix_mean = FALSE, 
   init_cov = NULL, 
+  lambda = 0,
   maxit = 100, 
   eps = 1e-6, 
   report = TRUE
@@ -427,21 +423,23 @@ FitMVN <- function(
   # Case of no missingness.
   if (!is_mis) {
     out <- FitMVNComplete(
-      data,
-      init_mean,
-      fix_mean
+      data = data,
+      init_mean = init_mean,
+      fix_mean = fix_mean,
+      lambda = lambda
     )
     
   # Case with missingness.
   } else {
     out <- FitMVNMiss(
-      data,
-      init_mean,
-      fix_mean,
-      init_cov,
-      maxit,
-      eps,
-      report
+      data = data,
+      init_mean = init_mean,
+      fix_mean = fix_mean,
+      init_cov = init_cov,
+      lambda = lambda,
+      maxit = maxit,
+      eps = eps,
+      report = report
     )
   }
   return(out)
